/*-
 * Copyright (c) 2007 Robert N. M. Watson
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <sys/types.h>
#include <sys/sysctl.h>
#include <sys/user.h>

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "procstat.h"

void
procstat_threads(pid_t pid)
{
	struct kinfo_proc *kip;
	const char *str;
	int i, name[4];
	size_t len;

	name[0] = CTL_KERN;
	name[1] = KERN_PROC;
	name[2] = KERN_PROC_PID | KERN_PROC_INC_THREAD;
	name[3] = pid;

	len = 0;
	if (sysctl(name, 4, NULL, &len, NULL, 0) < 0)
		err(-1, "sysctl: kern.proc.pid");

	kip = malloc(len);
	if (kip == NULL)
		err(-1, "malloc");

	if (sysctl(name, 4, kip, &len, NULL, 0) < 0)
		err(-1, "sysctl: kern.proc.pid");

	if (!hflag)
		printf("%5s %6s %2s %4s %-7s %-10s %-20s\n", "PID", "TID",
		    "CPU", "PRI", "STATE", "WCHAN", "COMM");
	for (i = 0; i < len / sizeof(*kip); i++) {
		printf("%5d ", pid);
		printf("%6d ", kip->ki_tid);
		printf("%3d ", kip->ki_oncpu);
		printf("%4d ", kip->ki_pri.pri_level);
		switch (kip->ki_stat) {
		case SRUN:
			str = "run";
			break;

		case SSTOP:
			str = "stop";
			break;

		case SSLEEP:
			str = "sleep";
			break;

		case SLOCK:
			str = "lock";
			break;

		case SWAIT:
			str = "wait";
			break;

		case SZOMB:
			str = "zomb";
			break;

		case SIDL:
			str = "idle";
			break;

		default:
			str = "??";
			break;
		}
		printf("%-7s ", str);
		if (kip->ki_kiflag & KI_LOCKBLOCK) {
			printf("*%-10s ", strlen(kip->ki_lockname) ?
			    kip->ki_lockname : "-");
		} else {
			printf("%-10s ", strlen(kip->ki_wmesg) ?
			    kip->ki_wmesg : "-");
		}
		printf("%-20s", strlen(kip->ki_ocomm) ? kip->ki_ocomm : "-");
		printf("\n");
	}
}
